max_denominator = 1000000
def gcd(a, b):
    while b:
        a, b = b, a % b
    return a

def extended_gcd(a,b):
    x = lasty = 0
    y = lastx = 1
    while b:
        quotient = a // b
        a, b = b, a % b
        x, lastx = lastx - quotient * x, x
        y, lasty = lasty - quotient * y, y
    return (lastx if a > 0 else -lastx, lasty if b > 0 else -lasty)

class Fraction(object):
    def __init__(self, numerator, denominator=1):
        if hasattr(denominator, '_numerator'):
            numerator *= denominator._denominator
            denominator = denominator._numerator
        if hasattr(numerator, '_numerator'):
            denominator *= numerator._denominator
            numerator = numerator._numerator
        
        self._numerator, self._denominator = numerator, denominator
        self._reduce()

    @property
    def numerator(self):
        return self._numerator

    @property
    def denominator(self):
        return self._denominator
    
    def __add__(self, other):
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return Fraction(self._numerator * other._denominator + other._numerator*self._denominator,
                        self._denominator * other._denominator)

    __radd__ = __add__

    def __mul__(self, other):
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return Fraction(self._numerator * other._numerator,
                        self._denominator * other._denominator)
    __rmul__ = __mul__
    
    def __div__(self, other):
        """ could use __mul__ but done directly for efficacy """
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return Fraction(self._numerator * other._denominator,
                        self._denominator * other._numerator)

    __truediv__ = __div__ 
    __floordiv__ = __div__

    def __rdiv__(self, other):
        """ could use __mul__ but done directly for efficacy """
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return Fraction(other._numerator * self._denominator,
                        other._denominator * self._numerator)

    __rfloordiv__ = __rdiv__
    __rtruediv__  = __rdiv__

    def __mod__(self, other):
        """self % other"""
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        div = int(self / other)
        return self - other * div

    def __rmod__(self, other):
        """other % self"""
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        div = int(other / self)
        return other - self * div

    def __pow__(self, other):
        """self ** other"""
        return (Fraction(int(self._numerator ** int(other)), int(self._denominator ** int(other))) if int(other) == other
                else (Fraction(self._numerator ** other, self._denominator ** other)))
    
    def __rpow__(self, other):
        """other ** self"""
        return other ** int(self) if self.is_integer() else other ** float(self)

    def __sub__(self, other):
        """ could use __add__ but done directly for efficacy """
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return Fraction(self._numerator * other._denominator - other._numerator * self._denominator,
                        self._denominator * other._denominator)

    def __rsub__(self, other):
        """ could use __add__ but done directly for efficacy """
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return Fraction(other._numerator * self._denominator - self._numerator * other._denominator,
                        other._denominator * self._denominator)

    def __neg__(self):
        return Fraction(-self._numerator, self._denominator)
    
    def _reduce(self):
        if self._numerator == self._denominator == 0:
            return
        denom = gcd(self._numerator, self._denominator)
        self._numerator = int(self._numerator / denom)
        self._denominator = int(self._denominator / denom)

    def  __eq__(self, other):
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return (self._numerator == other._numerator and
                self._denominator == other._denominator)

    def __ge__(self, other):
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return (self - other).numerator >= 0
        
    def __gt__(self, other):
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return (self - other).numerator > 0

    def __le__(self, other):
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return (self - other).numerator <= 0        
        
    def __lt__(self, other):
        if not hasattr(other, '_numerator'):
            other = Fraction(other)
        return (self - other).numerator < 0

    def __abs__(self):
        """ reduced fraction sign is in numerator """
        return Fraction(abs(self._numerator), self._denominator)

    def __float__(self):
        """ for float do floating point division """
        return float(self._numerator) / self._denominator

    def __trunc__(self):
        """ for int do integer division """
        return self._numerator // self._denominator
        
    def __str__(self):
        return "%i/%i" % (self._numerator, self._denominator)

    def __repr__(self):
        return "Fraction(%i, %i)" % (self._numerator, self._denominator)

    def is_integer(self):
        return self.denominator == 1

    def limit_denominator(self, max_denominator=max_denominator):
        """Closest Fraction to self with denominator at most max_denominator.

        >>> Fraction('3.141592653589793').limit_denominator(10)
        Fraction(22, 7)
        >>> Fraction('3.141592653589793').limit_denominator(100)
        Fraction(311, 99)
        >>> Fraction(4321, 8765).limit_denominator(10000)
        Fraction(4321, 8765)

        """
        # taken from Fractions module of Python 3.2
        # Algorithm notes: For any real number x, define a *best upper
        # approximation* to x to be a rational number p/q such that:
        #
        #   (1) p/q >= x, and
        #   (2) if p/q > r/s >= x then s > q, for any rational r/s.
        #
        # Define *best lower approximation* similarly.  Then it can be
        # proved that a rational number is a best upper or lower
        # approximation to x if, and only if, it is a convergent or
        # semiconvergent of the (unique shortest) continued fraction
        # associated to x.
        #
        # To find a best rational approximation with denominator <= M,
        # we find the best upper and lower approximations with
        # denominator <= M and take whichever of these is closer to x.
        # In the event of a tie, the bound with smaller denominator is
        # chosen.  If both denominators are equal (which can happen
        # only when max_denominator == 1 and self is midway between
        # two integers) the lower bound---i.e., the floor of self, is
        # taken.

        if max_denominator < 1:
            raise ValueError("max_denominator should be at least 1")
        if self._denominator <= max_denominator:
            return self

        p0, q0, p1, q1 = 0, 1, 1, 0
        n, d = self._numerator, self._denominator
        while True:
            a = n//d
            q2 = q0+a*q1
            if q2 > max_denominator:
                break
            p0, q0, p1, q1 = p1, q1, p0+a*p1, q2
            n, d = d, n-a*d

        k = (max_denominator-q0)//q1
        bound1 = Fraction(p0+k*p1, q0+k*q1)
        bound2 = Fraction(p1, q1)
        if abs(bound2 - self) <= abs(bound1-self):
            return bound2
        else:
            return bound1

if __name__=='__main__':
    print (Fraction(2.12,213.1).limit_denominator())
    print (Fraction(2,3) % 4)
    for a,b in ((Fraction(12,36),  Fraction(-1,-4)),
#{
                (0, Fraction(4,5)), (3, Fraction(9,3))
#}
        ):
        print('a: %s, b: %s' % (a,b))
        print(30*'-')
        print('%s == %s' % (a + b , b + a))
        print('- %s, %s' % (a - b , b - a))
        print('%s == %s' % (a * b , b * a))
        print('%s == %s' % (a + b , b + a))
        print('/ %s' % (a % b))
        print('** %s, %s' % (a ** b , b ** a))       
        print('%s == %s' % (a == b , b == a))
        print('%s == %s' % (a != b , b != a))
        print('%s == %s' % (a <= b , b >= a))
        print('%s == %s' % (a >= b , b <= a))
        print('%s == %s' % (a < b , b > a))
        print('Neg: %s, %s) % (-a, -b))
        print('Sum: %s' % (sum([a, b])))
        print('int: %s, %s' % (int(a), int(b)))
        print('float: %s, %s' % (float(a), float(b)))
        try:
            print('is_integer: %s, %s' % (a.is_integer(), b.is_integer()))
        except Exception as e:
            print(e)
        print('')
